@@ -96,7 +96,7 @@ static const char *clazzClassScript =
 "::oo::class create ::itcl::clazz {\n"
 "  superclass ::oo::class\n"
 "  method unknown args {\n"
-"    tailcall ::itcl::parser::handleClass [lindex [info level 0] 0] [self] {*}$args\n"
+"    ::tailcall ::itcl::parser::handleClass [::lindex [::info level 0] 0] [self] {*}$args\n"
 "  }\n"
 "  unexport create new unknown\n"
 "}";
@@ -165,7 +165,7 @@ RootCallProc(
     ItclObject *ioPtr = Tcl_ObjectGetMetadata(oPtr, &objMDT);
     ItclRootMethodProc *proc = (ItclRootMethodProc *)clientData;
 
-    return (*proc)(ioPtr, interp, objc+1, objv-1);
+    return (*proc)(ioPtr, interp, objc, objv);
 }
 
 /*
@@ -229,13 +229,6 @@ Initialize (
         Tcl_Panic("Itcl: cannot create namespace: \"%s\" \n", ITCL_NAMESPACE);
     }
 
-    nsPtr = Tcl_CreateNamespace(interp, ITCL_NAMESPACE"::methodset",
-            NULL, NULL);
-    if (nsPtr == NULL) {
-        Tcl_Panic("Itcl: cannot create namespace: \"%s::methodset\" \n",
-	        ITCL_NAMESPACE);
-    }
-
     nsPtr = Tcl_CreateNamespace(interp, ITCL_NAMESPACE"::internal::dicts",
             NULL, NULL);
     if (nsPtr == NULL) {
@@ -260,13 +253,6 @@ Initialize (
 #endif
     /* END for debugging only !!! */
 
-    Tcl_CreateObjCommand(interp,
-            ITCL_NAMESPACE"::methodset::callCCommand",
-            ItclCallCCommand, NULL, NULL);
-    Tcl_CreateObjCommand(interp,
-            ITCL_NAMESPACE"::methodset::objectUnknownCommand",
-            ItclObjectUnknownCommand, NULL, NULL);
-
     /*
      *  Create the top-level data structure for tracking objects.
      *  Store this as "associated data" for easy access, but link
@@ -290,8 +276,8 @@ Initialize (
     Tcl_InitObjHashTable(&infoPtr->nameClasses);
     Tcl_InitHashTable(&infoPtr->namespaceClasses, TCL_ONE_WORD_KEYS);
     Tcl_InitHashTable(&infoPtr->procMethods, TCL_ONE_WORD_KEYS);
-    Tcl_InitObjHashTable(&infoPtr->instances);
-    Tcl_InitHashTable(&infoPtr->objectInstances, TCL_ONE_WORD_KEYS);
+    Tcl_InitHashTable(&infoPtr->instances, TCL_STRING_KEYS);
+    Tcl_InitHashTable(&infoPtr->frameContext, TCL_ONE_WORD_KEYS);
     Tcl_InitObjHashTable(&infoPtr->classTypes);
     infoPtr->ensembleInfo = (EnsembleInfo *)ckalloc(sizeof(EnsembleInfo));
     memset(infoPtr->ensembleInfo, 0, sizeof(EnsembleInfo));
@@ -343,7 +329,6 @@ Initialize (
     }
     infoPtr->useOldResolvers = opt;
     Itcl_InitStack(&infoPtr->clsStack);
-    Itcl_InitStack(&infoPtr->contextStack);
 
     Tcl_SetAssocData(interp, ITCL_INTERP_DATA,
         (Tcl_InterpDeleteProc*)FreeItclObjectInfo, (ClientData)infoPtr);
@@ -366,6 +351,9 @@ Initialize (
     Tcl_NewMethod(interp, Tcl_GetObjectAsClass(root),
 	    Tcl_NewStringObj("ItclConstructBase", -1), 0,
 	    &itclRootMethodType, ItclConstructGuts);
+    Tcl_NewMethod(interp, Tcl_GetObjectAsClass(root),
+	    Tcl_NewStringObj("info", -1), 1,
+	    &itclRootMethodType, ItclInfoGuts);
 
     /* first create the Itcl base class as root of itcl classes */
     if (Tcl_EvalEx(interp, clazzClassScript, -1, 0) != TCL_OK) {
@@ -516,72 +504,6 @@ Itcl_SafeInit (
     return Tcl_EvalEx(interp, safeInitScript, -1, 0);
 }
 
-/*
- * ------------------------------------------------------------------------
- *  ItclCallCCommand()
- *  syntax: is
- *  objv[0]    command name of myself (::itcl::methodset::callCCommand)
- * ------------------------------------------------------------------------
- */
-
-int
-ItclCallCCommand(
-    ClientData clientData,
-    Tcl_Interp *interp,
-    int objc,
-    Tcl_Obj *const *objv)
-{
-    Tcl_CmdProc *argProc;
-    Tcl_ObjCmdProc *objProc;
-    ClientData cData;
-    int result;
-
-    ItclShowArgs(2, "ItclCallCCommand", objc, objv);
-    if (!Itcl_FindC(interp, Tcl_GetString(objv[1])+1, &argProc,
-            &objProc, &cData)) {
-	Tcl_AppendResult(interp, "no such registered C command 1: \"",
-	        Tcl_GetString(objv[1]), "\"", NULL);
-        return TCL_ERROR;
-    }
-    if ((argProc == NULL) && (objProc == NULL)) {
-	Tcl_AppendResult(interp, "no such registered C command 2: \"",
-	        Tcl_GetString(objv[1]), "\"", NULL);
-        return TCL_ERROR;
-    }
-    result = TCL_ERROR;
-    if (argProc != NULL) {
-	const char **argv;
-	int i;
-
-	argv = (const char**)ckalloc((unsigned)((objc-1)*sizeof(char*)));
-	for (i=1;i<objc;i++) {
-	    argv[i-1] = Tcl_GetString(objv[i]);
-	}
-        result = (*argProc)(cData, interp, objc-1, argv);
-        ckfree((char*)argv);
-    }
-    if (objProc != NULL) {
-#ifdef FIXED_ITCL_CALL_CONTEXT
-	Tcl_Namespace *callerNsPtr;
-        ItclObjectInfo *infoPtr;
-        callerNsPtr = Itcl_GetUplevelNamespace(interp, 1);
-        ItclShowArgs(2, "CARGS", Itcl_GetCallFrameObjc(interp),
-	        Itcl_GetCallFrameObjv(interp));
-        infoPtr = (ItclObjectInfo *)Tcl_GetAssocData(interp,
-                ITCL_INTERP_DATA, NULL);
-
-/* FIXME have to use ItclCallContext here !!! */
-/*	Itcl_PushStack(callerNsPtr, &infoPtr->namespaceStack); */
-#endif
-        result = (*objProc)(cData, interp, Itcl_GetCallFrameObjc(interp)-1,
-	        Itcl_GetCallFrameObjv(interp)+1);
-#ifdef FIXED_ITCL_CALL_CONTEXT
-/*	Itcl_PopStack(&infoPtr->namespaceStack); */
-#endif
-    }
-    return result;
-}
-
 /*
  * ------------------------------------------------------------------------
  *  ItclSetHullWindowName()
@@ -766,6 +688,13 @@ ItclFinishCmd(
     }
     Tcl_DeleteHashTable(&infoPtr->classTypes);
 
+    Tcl_DeleteHashTable(&infoPtr->procMethods);
+
+    Tcl_DeleteHashTable(&infoPtr->objectCmds);
+    Tcl_DeleteHashTable(&infoPtr->classes);
+    Tcl_DeleteHashTable(&infoPtr->nameClasses);
+    Tcl_DeleteHashTable(&infoPtr->namespaceClasses);
+
     nsPtr = Tcl_FindNamespace(interp, "::itcl::parser", NULL, 0);
     if (nsPtr != NULL) {
         Tcl_DeleteNamespace(nsPtr);
@@ -823,10 +752,6 @@ ItclFinishCmd(
     if (nsPtr != NULL) {
         Tcl_DeleteNamespace(nsPtr);
     }
-    nsPtr = Tcl_FindNamespace(infoPtr->interp, "::itcl::methodset", NULL, 0);
-    if (nsPtr != NULL) {
-        Tcl_DeleteNamespace(nsPtr);
-    }
     nsPtr = Tcl_FindNamespace(infoPtr->interp, "::itcl::internal", NULL, 0);
     if (nsPtr != NULL) {
         Tcl_DeleteNamespace(nsPtr);
@@ -846,7 +771,6 @@ ItclFinishCmd(
     ckfree((char *)infoPtr->class_meta_type);
 
     Itcl_DeleteStack(&infoPtr->clsStack);
-    Itcl_DeleteStack(&infoPtr->contextStack);
     /* clean up list pool */
     Itcl_FinishList();
 
